import ast
from datetime import datetime
import json
from urllib.parse import urlparse

from scrapy import Selector


def get_images(images):
    if ";" in images:
        return images

    return ";".join(images.split(","))


def is_blank(value):
    blanks = ["various numbers", "ノーブランド", "various countries", "なし"]
    for blank in blanks:
        if blank in value:
            return True


def remove_invalid_specifications(specifications):
    if not specifications:
        return None
    specs = []
    for spec in specifications:
        if spec["value"] in ["無し", "-"]:
            continue
        specs.append(spec)
    return None if not specs else specs


def get_specifications(json_data):
    specs = []
    try:
        attrs = json_data["data"]["itemInfoSku"]["attributes"]
        for attr in attrs:
            if is_blank(attr["value"]):
                continue

            spec = {}
            spec["name"] = attr["title"]
            spec["value"] = attr["value"]
            specs.append(spec)
        specs = remove_invalid_specifications(specs)
        return None if not specs else specs
    except Exception as e:
        pass

    if not specs:
        return None

    new_specs = []
    if isinstance(specs, str):
        specs = ast.literal_eval(specs)

    for spec in specs:
        if spec["value"] == "-":
            continue
        new_specs.append(spec)

    return new_specs if new_specs else None


def get_product_url_path(url):
    if not url:
        return None

    parsed_url = urlparse(url)
    return parsed_url.path


def get_product_id(json_data):
    try:
        return json_data["data"]["itemInfoSku"]["itemId"]
    except KeyError as e:
        raise Exception("can not find product id")


def get_reviews(doc):
    if "reviews" in doc:
        return doc["reviews"]

    return doc.get("reviews count")


def get_has_only_default_variant(doc):
    keys = ["variants", "variations"]
    for key in keys:
        if doc.get(key):
            return False

    return True


def get_variants(doc):
    keys = ["variants", "variations"]
    for key in keys:
        if doc.get(key):
            return doc.get(key)


def is_invalid_option(option):
    invalid_option_keys = [
        "在庫",
        "では別途送料",
        "お取り寄せ品のため",
        "欠品等で完売している場合がございます",
        "商品のケース箱ではない場合がございます",
        "レス化でエコ活動を実施中",
        "商品到着後にレビューを書いて",
        "円OFFクーポ",
        "イベント対象商品",
        "付属品",
        "配送方法",
        "宅配便希望の方は",
        "他商品を一緒",
        "メール便は代引き",
        "メール便はラッピン",
        "お急ぎメール便",
        "宅配便ご希望時",
        "でアマゾン",
        "にレビュ",
        "登録番号付き領収書",
        "お届け希望",
        "長期不在",
        "包装",
        "ッセージカ",
        "着後",
        "送から",
    ]
    for key in invalid_option_keys:
        if key in option["key"]:
            return True


def get_options(doc):
    new_options = []
    original_options = []
    if isinstance(doc.get("options"), str):
        if not doc.get("options"):
            return None

        original_options = ast.literal_eval(doc.get("options"))

    for op in original_options:
        if is_invalid_option(op):
            continue
        new_options.append(op)

    return new_options if new_options else None


def to_standard(doc):
    if not doc["images"] or not doc.get("url", doc.get("product_url")):
        return

    if not doc.get("product_id"):
        return None

    product = {
        "date": doc.get("date", datetime.now().replace(microsecond=0).isoformat()),
        "url": doc.get("url", doc.get("product_url")),
        "source": "Rakuten",
        "product_id": doc.get("product_id"),
        "existence": doc.get("existence"),
        "title": doc["title"],
        "title_en": doc.get("title_en"),
        "description": doc["description"],
        "summary": None,
        "sku": f"X01_{doc.get('product_id')}",
        "upc": doc.get("upc"),
        "brand": doc.get("brand"),
        "specifications": doc.get("specifications"),
        "categories": doc.get("categories", doc.get("category")),
        "images": get_images(doc["images"]),
        "videos": None,
        # "price": round(float(doc["price"]) * 0.0063, 2) if "stock status" in doc else doc["price"],
        "price": round(float(doc["price"]) * 0.007, 2),
        "available_qty": doc["available_qty"],
        # "options": get_options(doc),
        "options": doc["options"],
        # "variants": get_variants(doc),
        "variants": None,
        "returnable": False,
        "reviews": get_reviews(doc),
        "rating": doc["rating"],
        "sold_count": None,
        "shipping_fee": 3,
        "shipping_days_min": None,
        "shipping_days_max": None,
        "weight": doc.get("weight"),
        "width": doc.get("width"),
        "height": doc.get("height"),
        "length": doc.get("length"),
        "has_only_default_variant": get_has_only_default_variant(doc),
        "description_en": None,
    }
    return product


def get_price(json_data):
    return (
        json_data.get("data", {})
        .get("itemInfoSku", {})
        .get("purchaseInfo", {})
        .get("purchaseBySellType", {})
        .get("normalPurchase", {})
        .get("preTaxPrice")
    )


def get_existence(json_data):
    if "sold-out" in json_data.get("data", {}).get("itemInfoSku", {}).get("purchaseInfo", {}).get(
        "newPurchaseSku", {}
    ).get("stockCondition", ""):
        return False
    return bool(
        json_data.get("data", {})
        .get("itemInfoSku", {})
        .get("purchaseInfo", {})
        .get("newPurchaseSku", {})
        .get("quantity")
    )


def get_available_qty(json_data):
    try:
        return int(
            json_data.get("data", {})
            .get("itemInfoSku", {})
            .get("features", {})
            .get("inventoryDisplay")
        )
    except ValueError as e:
        pass


def get_options_from_data(json_data):
    unique_options = set()
    axises = (
        json_data.get("data", {}).get("itemInfoSku", {}).get("purchaseInfo", {}).get("axis", [])
    )
    for axis in axises:
        if is_invalid_option(axis):
            continue
        unique_options.add(axis["key"])

    if unique_options:
        return [{"name": option} for option in unique_options]


class ProductParser:
    def get_category_urls(self, response):
        return response.css(".dui-card>div.title>a::attr(href)").getall()

    def get_main_category_urls(self, response):
        return response.css("a.gtc-searchGenreCard__itemLink.gtu-link::attr(href)").getall()

    def is_blank(self, value):
        blanks = ["various numbers", "ノーブランド", "various countries", "なし"]
        for blank in blanks:
            if blank in value:
                return True

    def parse(self, response):
        item = {}
        if "category" in response.meta:
            item["category"] = response.meta.get("category")
        if "reviews" in response.meta:
            item["reviews"] = response.meta.get("reviews")

        if "rating" in response.meta:
            item["rating"] = response.meta.get("rating")

        if "rating" in response.meta:
            item["rating"] = response.meta.get("rating")

        api_datas = list(
            filter(lambda res: res.re_first('"api": (.+),'), response.css("script::text"))
        )
        if not api_datas:
            return
        json_data = api_datas[0]

        json_data = json.loads(response.css("script::text").re_first('"api": (.+),'))
        des = (
            Selector(
                text=json_data.get("data", {})
                .get("itemInfoSku", {})
                .get("pcFields", {})
                .get("productDescription", "")
            )
            .css("*::text")
            .getall()
        )
        desc = " ".join([val.strip() for val in des if val.strip()])
        variations = list()
        for var in json_data.get("data", {}).get("itemInfoSku", {}).get("sku", []):
            variations.append(
                {
                    "variant_id": var.get("variantId"),
                    "images": var.get("images", []),
                    "price": var.get("taxIncludedPrice"),
                }
            )

        doc = {
            "title": json_data.get("data", {}).get("itemInfoSku", {}).get("title"),
            "description": desc,
            "images": ";".join(response.xpath('//div[@irc="Image"]/../meta/@content').getall()),
            "price": get_price(json_data),
            "upc": json_data.get("data", {}).get("itemInfoSku", {}).get("itemId", ""),
            "available_qty": get_available_qty(json_data),
            "shipping_days_min": None,
            "shipping_days_max": None,
            "options": get_options_from_data(json_data),
            "variations": variations,
            "product_url": response.url,
            "product_id": get_product_id(json_data),
            "specifications": get_specifications(json_data),
            "existence": get_existence(json_data),
            "date": datetime.now().replace(microsecond=0).isoformat(),
        }
        item.update(doc)
        return to_standard(item)


class CategoryParser:
    def parse(self, response):
        pass

    def get_category_urls(self, response):
        return response.css(".dui-card>div.title>a::attr(href)").getall()

    def get_main_category_urls(self, response):
        return response.css("a.gtc-searchGenreCard__itemLink.gtu-link::attr(href)").getall()
