from datetime import datetime, timedelta
import json
from multiprocessing.pool import ThreadPool
from langdetect import detect , LangDetectException

from lingua import Language, LanguageDetectorBuilder
from google.oauth2 import service_account
from google.cloud.translate_v2 import Client


from utils.site_product import fetch_source_ids
from utils.translate_service import get_translator

from .loader import TranslationElasticSearchLoder
from rakuten.settings import (
    ELASTICSEARCH_SERVERS,
    ELASTICSEARCH_USERNAME,
    ELASTICSEARCH_PASSWORD,
)


def translate_product_title(product, title_translator):
    if product.get("title_en"):
        return product.get("title_en")
    return title_translator(product.get("title"))


def update_product_translated_title(standard, product, title_translator):
    translated_title = translate_product_title(product, title_translator)
    standard.update_one({"_id": product["_id"]}, {"$set": {"title_en": translated_title}})


def translate_products(products, loader, translate_file, title_translator, upload_file):
    translation = loader.search_content(list(products.keys()), "translation")
    if translation:
        for key in translation:
            products[key]["title_en"] = translation[key][0]["title_en"]
            json.dump(products[key], upload_file, ensure_ascii=False)
            upload_file.write("\n")
            products.pop(key)

    if products:
        for id, product in products.items():
            try:
                translation = {
                    "id": str(id),
                    "title": product["title"],
                    "title_en": title_translator(product["title"]),
                }
                translate_file.write(json.dumps(translation) + "\n")
                loader.save_item(translation)

                product["title_en"] = translation["title_en"]
                json.dump(product, upload_file, ensure_ascii=False)
                upload_file.write("\n")
            except Exception as e:
                print(e)


def is_valid(product) -> bool:
    if "date" not in product:
        return False

    date_obj = datetime.strptime(product["date"], "%Y-%m-%dT%H:%M:%S")
    return datetime.now() - date_obj < timedelta(weeks=2)


def main():
    service_account_path = "./gc.json"
    target_language = "en"
    translation_service = Client(
        target_language=target_language,
        credentials=service_account.Credentials.from_service_account_file(service_account_path),
    )

    pool = ThreadPool(10)

    count_total = 0
    title_translator = get_translator(translation_service, 30000, ["ja"], "title")

    product_ids = fetch_source_ids(["Rakuten"])


    filterd_settings = {
        "ELASTICSEARCH_SERVERS": ELASTICSEARCH_SERVERS,
        "ELASTICSEARCH_USERNAME": ELASTICSEARCH_USERNAME,
        "ELASTICSEARCH_PASSWORD": ELASTICSEARCH_PASSWORD,
    }

    translation_loader = TranslationElasticSearchLoder(filterd_settings, "id")

    batch_size = 500

    products = {}
    count = 0
    languages = [Language.JAPANESE, Language.ENGLISH]
    detector = LanguageDetectorBuilder.from_languages(*languages).build()
    with open("rakuten_upload_092001.txt", "w+") as upload_file:
        with open("rakuten_to_translate_0919.txt", "a+") as translate_file:
            with open("rakuten_upload_0919.txt", "r") as file:
                for line in file:
                    count += 1
                    product = json.loads(line)
                    product["product_id"] = str(product["product_id"])
                    if product["product_id"] in product_ids:
                        continue

                    if not is_valid(product):
                        continue

                    if not product.get("title"):
                        continue

                    product["title"] = product["title"].replace("[送料無料]", "").replace("[楽天海外直送]", "").replace("【", "").replace("】", "").replace("送料無料", "").replace("正規品", "").replace("楽天海外直送", "")


                    language = detector.detect_language_of(product["title"])
                    if language == Language.ENGLISH:
                        json.dump(product, upload_file, ensure_ascii=False)
                        upload_file.write("\n")
                        continue

                    if len(products) < batch_size:
                        products[product["product_id"]] = product
                        continue

                    translate_products(products, translation_loader, translate_file, title_translator, upload_file)
                    products = {}

                if products:
                    translate_products(products, translation_loader, translate_file, title_translator, upload_file)
                    products = {}

        # try:
        #     pool.apply_async(
        #         update_product_translated_title, args=(standard, product, title_translator)
        #     )
        # except Exception as e:
        #     pass

    pool.close()
    pool.join()

    print(count)


if __name__ == "__main__":
    main()
